@@ -5,21 +5,49 @@ from typing import Any, Dict, Optional, Tuple, List, Union, ClassVar
 # from pydantic.dataclasses import dataclass
 import re
 from datetime import date
+from functools import lru_cache
 from requests import Session, get
 from ._lib_wrapper.dataclass import dataclass
 from .utils.datetimes import parse_period_str, parse_weeks_str, parse_weekday_str, date_from_str
 from .mycqu import MycquUnauthorized
 
+
 __all__ = ("CQUSession", "CQUSessionInfo",
            "CourseTimetable", "CourseDayTime", "Course")
 
-
 CQUSESSIONS_URL = "https://my.cqu.edu.cn/api/timetable/optionFinder/session?blankOption=false"
 CUR_SESSION_URL = "https://my.cqu.edu.cn/api/resourceapi/session/cur-active-session"
+ALL_SESSIONSINFO_URL = "https://my.cqu.edu.cn/api/resourceapi/session/list"
 TIMETABLE_URL = "https://my.cqu.edu.cn/api/timetable/class/timetable/student/table-detail"
 
 
-@dataclass(order=True)
+def get_course_raw(session: Session, code: str, cqu_session: Optional[Union[CQUSession, str]] = None):
+    """从 my.cqu.edu.cn 上获取学生或老师的课表
+
+    :param session: 登录了统一身份认证（:func:`.auth.login`）并在 mycqu 进行了认证（:func:`.mycqu.access_mycqu`）的 requests 会话
+    :type session: Session
+    :param code: 学生或教师的学工号
+    :type code: str
+    :param cqu_session: 需要获取课表的学期，留空获取当前年级的课表
+    :type cqu_session: Optional[Union[CQUSession, str]], optional
+    :raises MycquUnauthorized: 若会话未在 my.cqu.edu.cn 进行认证
+    :return: 反序列化获取课表的json
+    :rtype: List[CourseTimetable]
+    """
+    if cqu_session is None:
+        cqu_session = CQUSessionInfo.fetch(session).session
+    elif isinstance(cqu_session, str):
+        cqu_session = CQUSession.from_str(cqu_session)
+    resp = session.post(TIMETABLE_URL,
+                        params={"sessionId": cqu_session.get_id()},
+                        json=[code],
+                        )
+    if resp.status_code == 401:
+        raise MycquUnauthorized()
+    return resp.json()['classTimetableVOList']
+
+
+@dataclass(order=True, frozen=True)
 class CQUSession:
     """重大的某一学期
     """
@@ -28,14 +56,14 @@ class CQUSession:
     is_autumn: bool
     """是否为秋冬季学期"""
     SESSION_RE: ClassVar = re.compile("^([0-9]{4})年?(春|秋)$")
-    CQUSESSION_MIN: ClassVar[CQUSession]
-    """my.cqu.edu.cn 支持的最早学期"""
+    _SPECIAL_IDS: ClassVar[Tuple[int, ...]] = (239259, 102, 101, 103, 1028, 1029, 1030, 1032) # 2015 ~ 2018
 
-    def __post_init_post_parse__(self):
-        if hasattr(CQUSession, "CQUSESSION_MIN"):
-            if self < CQUSession.CQUSESSION_MIN:
-                raise ValueError(
-                    f"session should not be earlier than {CQUSession.CQUSESSION_MIN}")
+    @lru_cache(maxsize=32)  # 这个缓存可以管 16 年的学期
+    def __new__(cls, year: int, is_autumn: bool):
+        return super(CQUSession, cls).__new__(cls)
+
+    def __str__(self):
+        return str(self.year) + ('秋' if self.is_autumn else '春')
 
     def get_id(self) -> int:
         """获取该学期在 my.cqu.edu.cn 中的 id
@@ -46,7 +74,12 @@ class CQUSession:
         :return: 学期的 id
         :rtype: int
         """
-        return (self.year-1503)*2 + int(self.is_autumn) + 1
+        if self.year >= 2019:
+            return (self.year - 1503) * 2 + int(self.is_autumn) + 1
+        elif 2015 <= self.year <= 2018:
+            return self._SPECIAL_IDS[(self.year - 2015) * 2 + int(self.is_autumn)]
+        else:
+            return (2015 - self.year) * 2 - int(self.is_autumn)
 
     @staticmethod
     def from_str(string: str) -> CQUSession:
@@ -81,19 +114,13 @@ class CQUSession:
         """
         session_list = []
         for session in get(CQUSESSIONS_URL).json():
-            try:
-                session_list.append(CQUSession.from_str(session["name"]))
-            except ValueError:
-                pass
+            session_list.append(CQUSession.from_str(session["name"]))
         return session_list
 
 
-CQUSession.CQUSESSION_MIN = CQUSession(2020, True)
-
-
 @dataclass
 class CQUSessionInfo:
-    """某学期的一些额外信息，目前只找到获取当个学期这些信息的 web api
+    """某学期的一些额外信息
     """
     session: CQUSession
     """对应的学期"""
@@ -102,6 +129,41 @@ class CQUSessionInfo:
     end_date: date
     """学期的结束日期"""
 
+    @staticmethod
+    def from_dict(data: dict[str, Any]) -> CQUSessionInfo:
+        """从反序列化的（一个）学期信息 json 中获取学期信息
+
+        :param data: json 反序列化得到的字典
+        :type data: dict[str, Any]
+        :return: 学期信息对象
+        :rtype: CQUSessionInfo
+        """
+        return CQUSessionInfo(
+            session=CQUSession(year=data["year"],
+                               is_autumn=data["term"] == "秋"),
+            begin_date=date_from_str(data["beginDate"]),
+            end_date=date_from_str(data["endDate"])
+        )
+
+    @staticmethod
+    def fetch_all(session: Session) -> List[CQUSessionInfo]:
+        """获取所有学期信息
+
+        :param session: 登录了统一身份认证（:func:`.auth.login`）并在 mycqu 进行了认证（:func:`.mycqu.access_mycqu`）的 requests 会话
+        :type session: Session
+        :return: 按时间降序排序的学期（最新学期可能尚未到来，其信息准确度也无法保障！）
+        :rtype: List[CQUSessionInfo]
+        """
+        resp = session.get(ALL_SESSIONSINFO_URL)
+        if resp.status_code == 401:
+            raise MycquUnauthorized()
+        cqusesions: List[CQUSessionInfo] = []
+        for data in resp.json()['sessionVOList']:
+            if not data['beginDate']:
+                break
+            cqusesions.append(CQUSessionInfo.from_dict(data))
+        return cqusesions
+
     @staticmethod
     def fetch(session: Session) -> CQUSessionInfo:
         """从 my.cqu.edu.cn 上获取当前学期的学期信息，需要登录并认证了 mycqu 的会话
@@ -115,13 +177,7 @@ class CQUSessionInfo:
         resp = session.get(CUR_SESSION_URL)
         if resp.status_code == 401:
             raise MycquUnauthorized()
-        data = resp.json()["data"]
-        return CQUSessionInfo(
-            session=CQUSession(year=data["year"],
-                               is_autumn=data["term"] == "秋"),
-            begin_date=date_from_str(data["beginDate"]),
-            end_date=date_from_str(data["endDate"])
-        )
+        return CQUSessionInfo.from_dict(resp.json()["data"])
 
 
 @dataclass
@@ -162,8 +218,8 @@ class Course:
     """课程代码"""
     course_num: Optional[str]
     """教学班号，在无法获取时（如考表 :class:`.exam.Exam` 中）设为 :obj:`None`"""
-    dept: str
-    """开课学院"""
+    dept: Optional[str]
+    """开课学院， 在无法获取时（如成绩 :class:`.score.Score`中）设为 :obj:`None`"""
     credit: Optional[float]
     """学分，无法获取到则为 :obj:`None`（如在考表 :class:`.exam.Exam` 中）"""
     instructor: Optional[str]
@@ -171,7 +227,7 @@ class Course:
     session: Optional[CQUSession]
     """学期，无法获取时则为 :obj:`None`"""
 
-    @ staticmethod
+    @staticmethod
     def from_dict(data: Dict[str, Any],
                   session: Optional[Union[str, CQUSession]] = None) -> Course:
         """从反序列化的（一个）课表或考表 json 中返回课程
@@ -192,8 +248,8 @@ class Course:
             code=data["courseCode"],
             course_num=data.get("classNbr"),
             dept=data.get(
-                "courseDepartmentName") or data["courseDeptShortName"],
-            credit=data.get("credit"),
+                "courseDepartmentName") or data.get("courseDeptShortName"),
+            credit=data.get("credit") or data.get("courseCredit"),
             instructor=data.get("instructorName"),
             session=session,
         )
@@ -237,7 +293,8 @@ class CourseTimetable:
         )
 
     @staticmethod
-    def fetch(session: Session, code: str, cqu_session: Optional[Union[CQUSession, str]] = None) -> List[CourseTimetable]:
+    def fetch(session: Session, code: str, cqu_session: Optional[Union[CQUSession, str]] = None)\
+            -> List[CourseTimetable]:
         """从 my.cqu.edu.cn 上获取学生或老师的课表
 
         :param session: 登录了统一身份认证（:func:`.auth.login`）并在 mycqu 进行了认证（:func:`.mycqu.access_mycqu`）的 requests 会话
@@ -250,16 +307,7 @@ class CourseTimetable:
         :return: 获取的课表对象的列表
         :rtype: List[CourseTimetable]
         """
-        if cqu_session is None:
-            cqu_session = CQUSessionInfo.fetch(session).session
-        elif isinstance(cqu_session, str):
-            cqu_session = CQUSession.from_str(cqu_session)
-        resp = session.post(TIMETABLE_URL,
-                            params={"sessionId": cqu_session.get_id()},
-                            json=[code],
-                            )
-        if resp.status_code == 401:
-            raise MycquUnauthorized()
-        return [CourseTimetable.from_dict(timetable) for timetable in resp.json()["classTimetableVOList"]
+        resp = get_course_raw(session, code, cqu_session)
+        return [CourseTimetable.from_dict(timetable) for timetable in resp
                 if timetable["teachingWeekFormat"]
                 ]
